/*
 * Copyright (c) 2009-2012 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * helloworld.c: simple test application
 *
 * This application configures UART 16550 to baud rate 9600.
 * PS7 UART (Zynq) is not initialized by this application, since
 * bootrom/bsp configures it to baud rate 115200
 *
 * ------------------------------------------------
 * | UART TYPE   BAUD RATE                        |
 * ------------------------------------------------
 *   uartns550   9600
 *   uartlite    Configurable only in HW design
 *   ps7_uart    115200 (configured by bootrom/bsp)
 */

#include <stdio.h>
#include <unistd.h>
#include "platform.h"
#include "xil_cache.h"

void print(char *str);

void dump(unsigned int *p, int len){
	int i;
	for(i = 0; i < len; i++){
		printf(" %08x", *(p+i));
		if(i % 4 == 3) printf("\n");
	}
	printf("\n");
}

void all_blue(){
	int i;
	for(i = 0; i < 1920*1080*3/4; i++){
		volatile unsigned int v = 0;

		switch(i % 3){
		case 0: v = 0xFF0000FF; break;
		case 1: v = 0x00FF0000; break;
		case 2: v = 0x0000FF00; break;
		default: v = 0;
		}
		*(((volatile unsigned int*)(0x3F000000))+i) = v;
	}
	Xil_DCacheFlushRange(0x3F000000,1920*1080*3);
}

void all_red_byte(){
	printf(" byte version\n");
	int i;
	for(i = 0; i < 1920*1080*3; i++){
		volatile unsigned char v = 0;
		if(i % 3 == 2) v = 0xFF;
		*(((volatile unsigned char*)(0x3F000000))+i) = v;
	}
	Xil_DCacheFlushRange(0x3F000000,1920*1080*3);
}

void all_red(){
	int i;
	for(i = 0; i < 1920*1080*3/4; i++){
		volatile unsigned int v = 0;
		switch(i % 3){
		case 0: v = 0x00FF0000; break;
		case 1: v = 0x0000FF00; break;
		case 2: v = 0xFF0000FF; break;
		default: v = 0;
		}
		*(((volatile unsigned int*)(0x3F000000))+i) = v;
	}
	Xil_DCacheFlushRange(0x3F000000,1920*1080*3);
}

void all_green(){
	int i;
	for(i = 0; i < 1920*1080*3/4; i++){
		volatile unsigned int v = 0;
		switch(i % 3){
		case 0: v = 0x0000FF00; break;
		case 1: v = 0xFF0000FF; break;
		case 2: v = 0x00FF0000; break;
		default: v = 0;
		}
		*(((volatile unsigned int*)(0x3F000000))+i) = v;
	}
	Xil_DCacheFlushRange(0x3F000000,1920*1080*3);
}

void all_clear(){
	int i;
	for(i = 0; i < 1920*1080*3/4; i++){
		volatile unsigned int v = 0;
		*(((volatile unsigned int*)(0x3F000000))+i) = v;
	}
	Xil_DCacheFlushRange(0x3F000000,1920*1080*3);
}

#define BASE (0x3F000000)
volatile unsigned char * FRAMEBUFFER = (volatile unsigned char*)BASE;

void pset(int x, int y, int rgb){
	unsigned char r = (rgb >> 16) & 0x000000FF;
	unsigned char g = (rgb >>  8) & 0x000000FF;
	unsigned char b = (rgb >>  0) & 0x000000FF;
	*(((volatile unsigned char*)(0x3F000000))+(3 * (y * 1920 + x) + 2)) = r;
	*(((volatile unsigned char*)(0x3F000000))+(3 * (y * 1920 + x) + 1)) = g;
	*(((volatile unsigned char*)(0x3F000000))+(3 * (y * 1920 + x) + 0)) = b;
	//printf("pset (%d, %d) = %d:%d:%d\n\r", x, y, r, g, b);
}

void fill_rect(int x, int y, int width, int height, int rgb){
	int i, j;
	for(i = 0; i < width; i++){
		for(j = 0; j < height; j++){
			pset(x+i, y+j, rgb);
		}
	}
}

int sintable[] = {
		500, 517, 543, 569, 595, 620, 637, 662, 687, 711, 734, 749, 772, 793, 814, 834,
		853, 865, 883, 899, 914, 928, 937, 949, 960, 969, 978, 982, 989, 993, 997, 999,
		999, 999, 998, 995, 990, 985, 980, 972, 963, 953, 941, 933, 919, 904, 888, 871,
		853, 840, 821, 800, 779, 757, 742, 719, 695, 671, 646, 629, 603, 578, 552, 526,
		500, 482, 456, 430, 404, 379, 362, 337, 312, 288, 265, 250, 227, 206, 185, 165,
		146, 134, 116, 100, 85, 71, 62, 50, 39, 30, 21, 17, 10, 6, 2, 0,
		0, 0, 1, 4, 9, 14, 19, 27, 36, 46, 58, 66, 80, 95, 111, 128,
		146, 159, 178, 199, 220, 242, 257, 280, 304, 328, 353, 370, 396, 421, 447, 473,
};

int colortbl[] = {0x00FF0000, 0x00FFFF00, 0x0000FF00, 0x0000FFFF, 0x000000FF, 0x00FF00FF};

void paint_sincurve(int offset){
	int c_id = 0;
	int i;
	for(i = 0; i < 1920/4; i++){
		int x = i * 4;
		int y = sintable[(i+offset)%128];
		int c = colortbl[c_id];
		fill_rect(x, y, 4, 4, c);
		c_id = c_id + 1;
		if(c_id == 6) c_id = 0;
	}
}


int main()
{
    init_platform();

    print("Hello World\n\r");

    printf("status = %08x\n\r", *((unsigned int*)(0xF8000900)));

    all_clear();

	*((unsigned int*)(0x41200000)) = 0x01;
#if 0
	fill_rect(0, 0, 1920, 360, 0x00FF0000);
	fill_rect(0, 360, 1920, 360, 0x0000FF00);
	fill_rect(0, 720, 1920, 360, 0x000000FF);

	Xil_DCacheFlushRange(0x3F000000,1920*1080*3);

    sleep(10);
    all_clear();

    paint_sincurve(0);
    paint_sincurve(32);
    paint_sincurve(64);
    paint_sincurve(96);
#endif

	Xil_DCacheFlushRange(0x3F000000,1920*1080*3);

    return 0;
}
