library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity conv64to24sim is
end conv64to24sim;

architecture RTL of conv64to24sim is

  component conv64to24
    port (
      clk : in std_logic;
      reset : in std_logic;
      fifo64_re : out std_logic;
      fifo64_din : in std_logic_vector(64-1 downto 0);
      fifo64_empty : in std_logic;
      fifo64_rd_count : in std_logic_vector(32-1 downto 0);
      fifo24_we : out std_logic;
      fifo24_dout : out std_logic_vector(24-1 downto 0);
      fifo24_full : in std_logic;
      fifo24_wr_count : in std_logic_vector(32-1 downto 0)
    );
  end component conv64to24;


  signal clk : std_logic := '0';
  signal reset : std_logic := '0';
  signal counter : signed(32-1 downto 0) := (others => '0');
  type Type_system is (
    system_IDLE,
    system_S0  
  );
  signal system : Type_system := system_IDLE;
  signal system_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0001 : signed(32-1 downto 0);
  signal tmp_0002 : std_logic;
  signal tmp_0003 : std_logic;
  signal tmp_0004 : std_logic;
  signal tmp_0005 : std_logic;
  signal U_clk : std_logic;
  signal U_reset : std_logic;
  signal U_fifo64_re : std_logic;
  signal U_fifo64_din : std_logic_vector(64-1 downto 0);
  signal U_fifo64_empty : std_logic := '0';
  signal U_fifo64_rd_count : std_logic_vector(32-1 downto 0);
  signal U_fifo24_we : std_logic;
  signal U_fifo24_dout : std_logic_vector(24-1 downto 0);
  signal U_fifo24_full : std_logic;
  signal U_fifo24_wr_count : std_logic_vector(32-1 downto 0) := (others => '0');
  signal fe : signed(64-1 downto 0);

begin


  -- expressions
  tmp_0001 <= counter + 1;
  tmp_0002 <= '1' when counter > 3 else '0';
  tmp_0003 <= '1' when counter < 8 else '0';
  tmp_0004 <= tmp_0002 and tmp_0003;
  tmp_0005 <= '1' when tmp_0004 = '1' else '0';

  -- sequencers
  process
  begin
    -- state system = system_IDLE
    system <= system_S0;
    wait for 10 ns;
    -- state system = system_S0
    system <= system_IDLE;
    wait for 10 ns;
  end process;


  process(system)
  begin
    if system = system_IDLE then
      clk <= '0';
    elsif system = system_S0 then
      clk <= '1';
    end if;
  end process;

  process(system)
  begin
    if system = system_IDLE then
      reset <= tmp_0005;
    end if;
  end process;

  process(system)
  begin
    if system = system_S0 then
      counter <= tmp_0001;
    end if;
  end process;

  U_clk <= clk;

  U_reset <= reset;

  U_fifo64_din <= X"0123456789abcdef";

  U_fifo64_rd_count <= X"00000014";

  U_fifo24_full <= '0';

  fe <= X"0123456789abcdef";


  inst_U : conv64to24
  port map(
    clk => U_clk,
    reset => U_reset,
    fifo64_re => U_fifo64_re,
    fifo64_din => U_fifo64_din,
    fifo64_empty => U_fifo64_empty,
    fifo64_rd_count => U_fifo64_rd_count,
    fifo24_we => U_fifo24_we,
    fifo24_dout => U_fifo24_dout,
    fifo24_full => U_fifo24_full,
    fifo24_wr_count => U_fifo24_wr_count
  );


end RTL;
