library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity sync_generator_sim is
end sync_generator_sim;

architecture RTL of sync_generator_sim is

  component sync_generator
    port (
      clk : in std_logic;
      reset : in std_logic;
      VSYNC : out std_logic;
      HSYNC : out std_logic;
      DE : out std_logic;
      DATA : out std_logic_vector(24-1 downto 0);
      fifo_rd : out std_logic;
      fifo_din : in std_logic_vector(24-1 downto 0);
      wakeup : in std_logic
    );
  end component sync_generator;


  signal clk : std_logic := '0';
  signal reset : std_logic := '0';
  signal counter : signed(32-1 downto 0) := (others => '0');
  type Type_system is (
    system_IDLE,
    system_S0  
  );
  signal system : Type_system := system_IDLE;
  signal system_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0001 : signed(32-1 downto 0);
  signal tmp_0002 : std_logic;
  signal tmp_0003 : std_logic;
  signal tmp_0004 : std_logic;
  signal tmp_0005 : std_logic;
  signal U_clk : std_logic;
  signal U_reset : std_logic;
  signal U_VSYNC : std_logic;
  signal U_HSYNC : std_logic;
  signal U_DE : std_logic;
  signal U_DATA : std_logic_vector(24-1 downto 0);
  signal U_fifo_rd : std_logic;
  signal U_fifo_din : std_logic_vector(24-1 downto 0) := (others => '0');
  signal U_wakeup : std_logic;
  signal tmp_0006 : std_logic;
  signal tmp_0007 : std_logic;

begin


  -- expressions
  tmp_0001 <= counter + 1;
  tmp_0002 <= '1' when counter > 3 else '0';
  tmp_0003 <= '1' when counter < 8 else '0';
  tmp_0004 <= tmp_0002 and tmp_0003;
  tmp_0005 <= '1' when tmp_0004 = '1' else '0';
  tmp_0006 <= '1' when counter > 20 else '0';
  tmp_0007 <= '1' when tmp_0006 = '1' else '0';

  -- sequencers
  process
  begin
    -- state system = system_IDLE
    system <= system_S0;
    wait for 10 ns;
    -- state system = system_S0
    system <= system_IDLE;
    wait for 10 ns;
  end process;


  process(system)
  begin
    if system = system_IDLE then
      clk <= '0';
    elsif system = system_S0 then
      clk <= '1';
    end if;
  end process;

  process(system)
  begin
    if system = system_IDLE then
      reset <= tmp_0005;
    end if;
  end process;

  process(system)
  begin
    if system = system_S0 then
      counter <= tmp_0001;
    end if;
  end process;

  U_clk <= clk;

  U_reset <= reset;

  U_wakeup <= tmp_0007;


  inst_U : sync_generator
  port map(
    clk => U_clk,
    reset => U_reset,
    VSYNC => U_VSYNC,
    HSYNC => U_HSYNC,
    DE => U_DE,
    DATA => U_DATA,
    fifo_rd => U_fifo_rd,
    fifo_din => U_fifo_din,
    wakeup => U_wakeup
  );


end RTL;
